/*
	autoresp Server for XFBB by Stewart Wilkinson G0LGS 

*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h> 
#include <time.h>
#include <sys/stat.h>

#include "utils.h"

FILE *init_ptr, *msg_ptr, *cfg_ptr, *out_ptr;

char buffer[255];

char from_call[10];
char from_bbs[80];

char subject[80];
char mail_in[128];
char sys_name[20];
char sys_call[10];
char bbs_call[80];

char sent_to[10];

char config[128];
char text[100];

#ifdef V700

#define TAIL "\n# [AUTORESP V" Version " " Vdate " for XFBB 7.00 (c) by G0LGS]\n"
#define INIT "init.srv"

int ReadInit()
{
	#define BBS_CALL 1
	#define SYSOP_NAME 11
	#define SYSOP_CALL 12
	#define MAIL_IN 14

	int end, index;

	/* Open init.srv */
 	init_ptr = fopen( INIT, "rt");
	if (init_ptr == NULL) {
		fprintf( stderr, "Unable to open '%s'\n", INIT );
		return 1; 
	}

	/* Read the lines we need . */
	index=end=0;
	while (! end && fgets(buffer, sizeof(buffer), init_ptr) && ! feof(init_ptr) ) {

		/* Comments ignored */
		if (*buffer == '#')
			continue;

		switch (++index) {

			case BBS_CALL :
				sscanf( buffer, "%s", bbs_call);
				/* We only want up to the first '.' */
				*strstr(bbs_call,".") = '\0';
				break;

			case SYSOP_NAME :
				sscanf( buffer, "%s", sys_name);
				break;

			case SYSOP_CALL :
				sscanf( buffer, "%s", sys_call);
				break;

			case MAIL_IN :
				sscanf( buffer, "%s", mail_in);
				end = 1;
				break;
		}
	}

	fclose(init_ptr);

	return 0;
}
#else

#define TAIL "\n# [AUTORESP V" Version " " Vdate " for XFBB 7.01+ (c) by G0LGS]\n"
#define FBBCONF "/etc/ax25/fbb.conf"

int ReadInit()
{
	int len, i;
	int end = 0;
	char buffer[256];
	char *scan;
	char *ptr;
	char key[20];
	char pathname[256];

	if( (ptr = getenv("FBBCONF")) == NULL ){
		strcpy( pathname, FBBCONF);
	}else{
		strcpy( pathname, ptr);
	}

	if ((init_ptr = fopen (pathname, "r")) != NULL) {

		while( !end && fgets(buffer, sizeof(buffer), init_ptr) && ! feof(init_ptr) ){

			scan = buffer;

			/* Eat Spaces */
			while (isspace(*scan))
				++scan;

			/* Comments & Blanks ignored */
			if (*scan == '#' || *scan == '\0' )
				continue;

			/* remove leading \n */
			len = strlen(scan);
			if( len && (scan[len-1] == '\n'))
				scan[len-1] = '\0';

			/* keyword */
			i=0;
			ptr = key;
			while( isgraph(*scan)) {
				if( i++ < 4 ) {
					*ptr++ = *scan;
				}
				++scan;
				if(*scan == '=')
					break;         
			}
			*ptr = '\0';

			/* find the '=' */
			scan = strchr(scan, '=');

			/* copy value */
			if(scan) {
				++scan;
				while( isspace(*scan))
					++scan;

				/* if 1st char is '"' then string */
				if( *scan == '"' ){
					++scan;
					ptr = strrchr(scan,'"');
					if(ptr)
						*ptr = '\0';
				}

				if( strcmp( key, "call" ) == 0 ) {
					strcpy( bbs_call, scan );
					*strstr( bbs_call, "." ) = '\0';

				}else if( strcmp( key, "name" ) == 0 ) {
					strcpy( sys_name, scan );

				}else if( strcmp( key, "syso" ) == 0 ) {
					strcpy( sys_call, scan );

				}else if( strcmp( key, "impo" ) == 0 ) {
					strcpy( mail_in, scan );

				}

			}
		}

		fclose(init_ptr);
  
	}else{
		fprintf( stderr, "Unable to open '%s'\n", pathname );
		return 1;
	}

	return 0;
}

#endif

int main(int argc, char **argv)
{

        char Cmd[128];
        char *ap;
	int end = 0;
	int msg_pos = 0;
	int index = 0;
	int Date, Time;

	/* Program Name */
        strcpy( Cmd, argv[0] );
        if( (ap = rindex( Cmd, '/') ) != NULL ) {
                ap++;
        }else{
                ap=Cmd;
        }

	/* Config file name */
	sprintf( config, "%s.msg", ap );

	/* Check the Arguments */
	if (argc != 2){
		fprintf( stderr, "%s", TAIL );
		exit(1);
	}

	if( ReadInit() != 0 ){
		fprintf( stderr, "Error Processing FBB Config file\n" );
		exit(1);
        }

	/* Open the Message */
	msg_ptr = fopen(argv[1], "r");
	if (msg_ptr == NULL){
		fprintf( stderr, "Cannot Open '%s'\n", argv[1] );
		exit(1);
	}

	/* Read the From line */
	fgets(buffer, sizeof(buffer), msg_ptr);
	sscanf(buffer, "%*s %s %*s %s", sent_to, from_call );

	/* Read the Subject */
	fgets(buffer, sizeof(buffer), msg_ptr);
	sscanf(buffer, "%[^\n]\n", subject);

	msg_pos = ftell( msg_ptr );

	/* Process any R: line to get originating BBS & Skip T: lines */
	end=0;
	while( ! end && fgets( buffer, sizeof(buffer), msg_ptr ) && ! feof(msg_ptr) ){

		/* R: lines */
		if( sscanf( buffer,"R:%d/%d%*[^@]@:%s", &Date, &Time, from_bbs ) >= 1 ) {
			msg_pos = ftell( msg_ptr );
			continue;

		/* T: lines */
		}else if( sscanf( buffer,"T:%[^\n]\n", text ) != 0 ) {
			msg_pos = ftell( msg_ptr );
			continue;

		/* Terminate when no R: or T: line */
		} else {
			end=1;
		}

	}


	/* Close the message file  */ 
	fclose(msg_ptr);
   
	/* Open config file */
	cfg_ptr = fopen( config, "rt" );
        if( cfg_ptr == NULL ) {
		fprintf( stderr, "Cannot Open '%s'\n", config );
		exit(1);
	}


	/* Send Response to the Originator */

	if( (out_ptr = fopen( mail_in, "a" )) == NULL ) {
		fprintf( stderr, "Unable to append to '%s'\n", mail_in );
		exit(1);
	}

	if( strlen(from_bbs) >= 6 ) {
		fprintf( out_ptr, "#\nSP %s @ %s < %s\n", from_call, from_bbs, bbs_call );
	}else{
		fprintf( out_ptr, "#\nSP %s < %s\n", from_call, bbs_call );
	}

	fprintf( out_ptr, "Re: %s\n", subject );

	fprintf( out_ptr, "\n%s\n", TAIL );
	fprintf( out_ptr, "\nRe Your: SP %s @ %s\n", strupr(ap), bbs_call);

	/* Process Text from config file */
	index=end=0;
	while( !end && fgets(buffer, sizeof(buffer), cfg_ptr) && ! feof(cfg_ptr) ){

		/* Comments ignored */
		if (*buffer == '#')
			continue;

		fprintf( out_ptr, buffer );
	}

	fclose(cfg_ptr);

	fprintf( out_ptr, "\n73s %s (%s) Sysop %s\n", sys_name, sys_call, bbs_call );

	fprintf( out_ptr, "\n%s", TAIL );
	fprintf( out_ptr, "/EX\n" );

	fclose( out_ptr );

	unlink(argv[1]);
	exit(0);
}

